/*  FUNNEL LIBRARY
**  
**  String -- support for strings in Funnel
**  
**  11/06/2K, Matthias Zenger
*/

type String = string

newtype StringModule = {
    def charAt(s: string)(i: int): char
    def length(s: string): int
    def startsWith(s: string)(p: string): boolean
    def endsWith(s: string)(p: string): boolean
    def indexOf(s: string)(i: int)(j: int): int
    def lastIndexOf(s: string)(i: int)(j: int): int
    def substring(s: string)(f: int)(t: int): string
    def replace(s: string)(f: char)(t: char): string
    def toLowerCase(s: string): string
    def toUpperCase(s: string): string
    def trim(s: string): string
    def toInt(s: string): int
    def toLong(s: string): long
    def toFloat(s: string): float
    def toDouble(s: string): double
    def from(x: any): string
    def fromChar(c: char): string
}

let String: StringModule = {
    def charAt (s: string) (i: int) : char =
      primitive (2, "java.lang.String", "charAt", "(I)C") (s, i)

    def length (s: string) : int = s.length

    def startsWith (s: string) (p: string) : boolean =
      primitive (2, "java.lang.String", "startsWith", "(Ljava/lang/String;)Z") (s, p)

    def endsWith (s: string) (p: string) : boolean =
      primitive (2, "java.lang.String", "endsWith", "(Ljava/lang/String;)Z") (s, p)

    def indexOf (s: string) (i: int) (j: int) : int =
      primitive (2, "java.lang.String", "indexOf", "(II)I") (s, i, j)

    def lastIndexOf (s: string) (i: int) (j: int) : int =
      primitive (2, "java.lang.String", "lastIndexOf", "(II)I") (s, i, j)

    def substring (s: string) (f: int) (t: int) : string =
      primitive (2, "java.lang.String", "substring", "(II)Ljava/lang/String;")
	(s, f, t)

    def replace (s: string) (f: char) (t: char) : string = 
      primitive (2, "java.lang.String", "replace", "(CC)Ljava/lang/String;")
	(s, f, t)

    def toLowerCase (s: string) : string =
      primitive (2, "java.lang.String", "toLowerCase", "()Ljava/lang/String;") (s)

    def toUpperCase (s: string) : string =
      primitive (2, "java.lang.String", "toUpperCase", "()Ljava/lang/String;") (s)

    def trim (s: string) : string =
      primitive (2, "java.lang.String", "trim", "()Ljava/lang/String;") (s)

    def toInt (s: string) : int =
      primitive (1, "java.lang.Integer", "parseInt", "(Ljava/lang/String;)I") (s)

    def toLong (s: string) : long =
      primitive (1, "java.lang.Long", "parseLong", "(Ljava/lang/String;)J") (s)

    def toFloat (s: string) : float =
      primitive (1, "java.lang.Float", "parseFloat", "(Ljava/lang/String;)F") (s)

    def toDouble (s: string) : double =
      primitive (1, "java.lang.Double", "parseDouble", "(Ljava/lang/String;)D") (s)

    def from (x : any) : string =
      x.toString

    def fromChar (x : char) : string =
      x.toString
}
